@@ -1,44 +1,11 @@
-
-## Take an M x J matrix of detection probabilities and return a matrix
-## of M x J observation probs
-# Compute the cell probabilities for the observation classes
-# in removal sampling.
-#
-# Both p and the returned matrix are M x J for M sites and J sampling occasions.
-
-removalPiFun <- function(p){
-  M <- nrow(p)
-  J <- ncol(p)
-  pi <- matrix(NA, M, J)
-  pi[,1] <- p[,1]
-  for(i in seq(from = 2, length = J - 1)) {
-	  pi[, i] <- pi[,i-1] / p[,i-1] * (1-p[,i-1]) * p[,i]
-  }
-  return(pi)
-}
-
-# p is an M x 2 matrix of detection probabilities (site x observer).
-# returns an M x 3 matrix of row=(1 not 2, 2 not 1, 1 and 2).
-# Compute the cell probabilities for the observation classes
-# in double observer sampling.
-
-doublePiFun <- function(p){
-  M <- nrow(p)
-  pi <- matrix(NA, M, 3)
-  pi[,1] <- p[,1] * (1 - p[,2])
-  pi[,2] <- p[,2] * (1 - p[,1])
-  pi[,3] <- p[,1] * p[,2]
-  return(pi)
-}
-
-
 # Fit the multinomial-Poisson abundance mixture model.
 
 multinomPois <- function(formula, data, starts, method = "BFGS",
-    se = TRUE, ...)
+    se = TRUE, engine = c("C","R"), ...)
 {
     if(!is(data, "unmarkedFrameMPois"))
 		    stop("Data is not a data frame or unmarkedFrame.")
+    engine <- match.arg(engine, c("C", "R"))
     designMats <- getDesign(data, formula)
     X <- designMats$X; V <- designMats$V; y <- designMats$y
     X.offset <- designMats$X.offset; V.offset <- designMats$V.offset
@@ -64,7 +31,7 @@ multinomPois <- function(formula, data, starts, method = "BFGS",
     yvec <- as.numeric(y)
     navec <- is.na(yvec)
 
-    nll <- function(parms) {
+    nll_R <- function(parms) {
         lambda <- exp(X %*% parms[1 : nAP] + X.offset)
         p <- plogis(V %*% parms[(nAP + 1) : nP] + V.offset)
         p.matrix <- matrix(p, M, R, byrow = TRUE)
@@ -73,6 +40,27 @@ multinomPois <- function(formula, data, starts, method = "BFGS",
         logLikeSite[navec] <- 0
         -sum(logLikeSite)
         }
+
+    nll_C <- function(params) {
+        .Call("nll_multinomPois",
+            params,piFun,
+            X, X.offset, V, V.offset,  
+            yC, navecC, nP,nAP,
+            PACKAGE = "unmarked")
+    }
+
+    if(engine=="R"){
+      nll <- nll_R
+    }else{
+      yC <- as.numeric(t(y))
+      navecC <- is.na(yC)
+      nll <- nll_C
+      if(!piFun%in%c('doublePiFun','removalPiFun','depDoublePiFun')){
+        warning("Custom pi functions are not supported by C engine. Using R engine instead.")
+        nll <- nll_R
+      }
+    }
+
     if(missing(starts))
         starts <- rep(0, nP)
     fm <- optim(starts, nll, method = method, hessian = se, ...)
